@@ -44,6 +44,11 @@
 
 #define NOTEBOOK "notebook"
 
+/* Forward declarations */
+static void properties_response (GtkDialog *dialog, int response, SeahorseWidget *swidget);
+static void properties_destroyed (GtkObject *object, SeahorseWidget *swidget);
+
+
 static void 
 show_glade_widget (SeahorseWidget *swidget, const gchar *name, gboolean show)
 {
@@ -220,10 +225,29 @@ static void
 names_delete_clicked (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	SeahorseKey *skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+	SeahorsePGPKey *pkey = SEAHORSE_PGP_KEY (skey);
 	gint index = names_get_selected_uid (swidget);
+	gboolean ret;
+	gchar *userid, *message; 
+	gpgme_error_t gerr;
     
-	if (index >= 1) 
-		seahorse_pgp_delete_userid_show (skey, index);
+	if (index < 1)
+		return;
+	
+	/* TODO: This whole index thing is messed up */
+	userid = seahorse_key_get_name (skey, index - 1);
+	message = g_strdup_printf (_("Are you sure you want to permanently delete the '%s' user ID?"), userid);
+	g_free (userid);
+	
+	ret = seahorse_util_prompt_delete (message, seahorse_widget_get_toplevel (swidget));
+	g_free (message);
+	
+	if (ret == FALSE)
+		return;
+	
+	gerr = seahorse_pgp_key_op_del_uid (pkey, index);
+	if (!GPG_IS_OK (gerr))
+		seahorse_pgp_handle_gpgme_error (gerr, _("Couldn't delete user ID"));
 }
 
 static void 
@@ -967,8 +991,26 @@ static void
 details_del_subkey_button_clicked (GtkButton *button, SeahorseWidget *swidget)
 {
 	SeahorseKey *skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+	SeahorsePGPKey *pkey = SEAHORSE_PGP_KEY (skey);
+	guint index = get_selected_subkey (swidget);
+	gboolean ret;
+	gchar *userid, *message; 
+	gpgme_error_t err;
 
-	seahorse_pgp_delete_subkey_new (SEAHORSE_PGP_KEY (skey), get_selected_subkey (swidget));
+	userid = seahorse_key_get_name (SEAHORSE_KEY (pkey), 0);
+	message = g_strdup_printf (_("Are you sure you want to permanently delete subkey %d of %s?"),
+	                           index, userid);
+	g_free (userid);
+	
+	ret = seahorse_util_prompt_delete (message, seahorse_widget_get_toplevel (swidget));
+	g_free (message);
+	
+	if (ret == FALSE)
+		return;
+	
+	err = seahorse_pgp_key_op_del_subkey (pkey, index);
+	if (!GPG_IS_OK (err))
+		seahorse_pgp_handle_gpgme_error (err, _("Couldn't delete subkey"));
 }
 
 static void
@@ -1137,7 +1179,7 @@ setup_details_trust (SeahorseWidget *swidget)
        
         gtk_list_store_append (model, &iter);
         gtk_list_store_set (model, &iter,
-                            TRUST_LABEL, _("Never"),
+                            TRUST_LABEL, C_("Validity","Never"),
                             TRUST_VALIDITY,  SEAHORSE_VALIDITY_NEVER,
                             -1);
     }
@@ -1269,7 +1311,7 @@ do_details (SeahorseWidget *swidget)
         if (seahorse_key_get_location (skey) == SEAHORSE_LOCATION_REMOTE)
             fp_label = NULL;
         else if (subkey->expires == 0)
-            fp_label = g_strdup (_("Never"));
+            fp_label = g_strdup (C_("Expires", "Never"));
         else
             fp_label = seahorse_util_get_display_date_string (subkey->expires);
         gtk_label_set_text (GTK_LABEL (widget), fp_label);
@@ -1337,11 +1379,15 @@ do_details (SeahorseWidget *swidget)
 
         subkey = seahorse_pgp_key_get_nth_subkey (pkey, key);
 
-        status = subkey->revoked ? _("Revoked") : _("Good");
-        status = subkey->expired ? _("Expired") : _("Good");
+        if (subkey->revoked)
+        	status = _("Revoked");
+        else if (subkey->expired)
+        	status = _("Expired");
+        else
+        	status = _("Good");
 
         if (subkey->expires == 0)
-            expiration_date = g_strdup (_("Never"));
+            expiration_date = g_strdup (C_("Expires", "Never"));
         else
             expiration_date = seahorse_util_get_display_date_string (subkey->expires);
 
@@ -1792,7 +1838,9 @@ key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseWidget *swidge
 static void
 key_destroyed (GtkObject *object, SeahorseWidget *swidget)
 {
-    seahorse_widget_destroy (swidget);
+	GtkWidget *widget = seahorse_widget_get_toplevel(swidget);
+	g_signal_handlers_disconnect_by_func (widget, properties_destroyed, swidget); 
+	g_signal_handlers_disconnect_by_func (widget, properties_response, swidget);
 }
 
 static void
@@ -1827,7 +1875,7 @@ setup_public_properties (SeahorsePGPKey *pkey, GtkWindow *parent)
     if (swidget == NULL)
         return NULL;
 
-    widget = glade_xml_get_widget (swidget->xml, swidget->name);
+    widget = seahorse_widget_get_toplevel(swidget);
     g_signal_connect (widget, "response", G_CALLBACK (properties_response), swidget);
     g_signal_connect (widget, "destroy", G_CALLBACK (properties_destroyed), swidget);
     g_signal_connect_after (pkey, "changed", G_CALLBACK (key_changed), swidget);
